/******************************************************************************/
/* (c) 2006 jamie twycross, jpt AT cs.nott.ac.uk                              */
/* released under gnu gpl v2                                                  */
/* tissue client api                                                          */
/******************************************************************************/

#include "tissue.h"
#include "client.h"
#include "misc.h"

#include <arpa/inet.h>
#include <netdb.h>
#include <netinet/in.h>
#include <netinet/sctp.h>
#include <sys/un.h>
#include <strings.h>
#include <unistd.h>
#include <err.h>
#include <stdlib.h>
#include <string.h>

/* tissue server socket + file descriptors */
static int sd = -1;
static char *clientname;

/* connect to tissue server */
void connect_ts(const char *addr, uint16_t port, const char *desc)
{
	struct hostent *he;
	struct sockaddr_in saddr;
	char message[strlen(desc) + 18];

	if(!(he = gethostbyname(addr)))
		err(1, "couldn't resolve tissue server name");
	if ( he->h_length != sizeof(saddr.sin_addr.s_addr) ) 
		err(1, "bad address");
	saddr.sin_family = AF_INET;
	saddr.sin_addr.s_addr = *((uint32_t *) he->h_addr);
	saddr.sin_port = htons(port);

	if((sd = socket(PF_INET, SOCK_STREAM, IPPROTO_SCTP)) < 0 )
		err(1, "couldn't create socket");
	if(connect(sd, (struct sockaddr *) &saddr, sizeof(saddr)) < 0 )
		err(1, "couldn't connect socket");

	if(!(clientname = strdup(desc)))
		err(1, "couldn't use client name");
	snprintf(message, sizeof(message), "%s client connected", clientname);
	send_message(message);
}

void connect_tslocal(const char *sockname, const char *desc)
{
	struct sockaddr_un sa;
	char message[strlen(desc) + 18];

	sa.sun_family = AF_UNIX;
	snprintf(sa.sun_path, sizeof(sa.sun_path), "%s", sockname);

	if((sd = socket(PF_UNIX, SOCK_STREAM, 0)) < 0 )
		err(1, "couldn't create socket");
	if(connect(sd, (struct sockaddr *) &sa, sizeof(sa)) < 0 )
		err(1, "couldn't connect socket");

	if(!(clientname = strdup(desc)))
		err(1, "couldn't use client name");
	snprintf(message, sizeof(message), "%s client connected", clientname);
	send_message(message);
}

/* disconnect from tissue server */
void disconnect_ts(void)
{
	static char message[64];

	if(sd == -1)
		return;
	snprintf(message, sizeof(message), "%s client disconnected", clientname);
	send_message(message);
	close(sd);
	sd = -1;
}

/* send datastream event to tissue server */
void send_ds(DS *ds)
{
	static char pkt[4096];
	int len;

	if(sd == -1)
		return;
	/* construct data packet to send */
	if((len = ds2pkt(ds, pkt, sizeof(pkt))) == -1)
		return;
	/* send data packet */
	if(send(sd, pkt, len, MSG_NOSIGNAL) != len)
		err(1, "couldn't send datastream");
}

/* send message to tissue server */
void send_message(char *message)
{
	static char pkt[4096];
	int len;

	if(sd == -1)
		return;
	/* construct data packet to send */
	pkt[0] = CLIENT_MSG;
	len = strlen(message);
	memcpy(pkt + 1, message, len + 1);
	/* send data packet */
	if(send(sd, pkt, len + 2, MSG_NOSIGNAL) != (len + 2))
		err(1, "couldn't send message");
}

void send_time_sync(void)
{
	static const uint8_t time_sync = CLIENT_TIME_SYNC;
	if ( send(sd, &time_sync, sizeof(time_sync), MSG_NOSIGNAL) != 1 )
		err(1, "couldn't send time sync");
}
