/******************************************************************************/
/* (c) 2006 jamie twycross, jpt AT cs.nott.ac.uk                              */
/* released under gnu gpl v2                                                  */
/* tissue api                                                                 */
/******************************************************************************/

#ifndef TISSUE_H
#define TISSUE_H

#include "misc.h"
#include "log.h"
#include "antigen.h"
#include "cytokine.h"
#include "cell.h"
#include "ds.h"
#include "client.h"
#include "response.h"
#include "compiler.h"
#include "list.h"
#include "nbio.h"

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <stdint.h>

#define TS_DEFAULT_PORT		5050  /* default port tissue server listens on */
#define TS_BUFFER_SIZE		0x10000  /* server input buffer size */

/* types for tissue cytokines */
#define CYTOKINE_REFLECTIVE		0  /* value replaces current tissue value */
#define CYTOKINE_ADDITIVE		1  /* value is added to current tissue value */
#define CYTOKINE_CUSTOM			2  /* tissue value function of current, value */

#define DEF_DECAY_RATE 		1.0  /* rate at which cytokines decay in tissue */
#define DEF_CYTOKINE_TYPE	CYTOKINE_REFLECTIVE  /* tissue cytokine type */

#define CLIENT_MSG		0x00
#define CLIENT_DS		0x01
#define CLIENT_TIME_SYNC	0x02

/* tissue composed of antigen, cytokines + cells */
typedef struct Tissue {
	unsigned int num_antigen;
	Antigen **antigen;
	unsigned int num_cytokines;
	Cytokine **cytokine;
	int *cytokine_type;
	unsigned int num_cells;
	Cell **cell;
	double cytokine_decay_rate;

	unsigned int antigen_multiplier;

	struct iothread nbio;
	struct nbio *remote_svr, *local_svr;
	unsigned int num_clients;
	struct list_head clients;
	uint64_t prev_time;

	/* flags */
	unsigned short time_sync:1;

	/* logging flags */
	unsigned short log:1;
	unsigned short log_antigen:1;  /* log tissue antigen */
	unsigned short log_cytokines:1;  /* log tissue cytokine */
	unsigned short log_cells:1;  /* log tissue cells */
	unsigned short log_tissue:1;  /* log all 3 of above */

	/* timers */
	struct list_head timers;
	unsigned int num_timers;
	unsigned int max_timers;
	struct itimer **prio;
} Tissue;

typedef void (*timer_cbfn_t)(Tissue *t);
struct itimer {
	uint64_t next_time;
	uint64_t interval;
	timer_cbfn_t cb;
	struct list_head list;
};

/* function prototypes */
Tissue *init_tissue(const uint16_t port, const char *sockname,
			const unsigned int num_antigen,
			const unsigned int num_cytokines,
			const unsigned int num_cells);
void add_timer(Tissue *t, uint64_t interval, timer_cbfn_t cb);
void ts_mainloop(Tissue *);
void free_tissue(Tissue *);
void print_tissue(Tissue *);
void print_tissue_cytokines(Tissue *);
void print_tissue_antigen(Tissue *);
void print_tissue_cells(Tissue *);
void set_cell_input(Tissue *, const unsigned int);
void set_tissue(Tissue *, const unsigned int);
void step_tissue(Tissue *);
int read_parameter_uint(char *, char *, unsigned int *);
void log_parameter_file(char *);

#endif
